'use strict';

function Trait(spec) {
    this.required = spec.required;
    this.exports = spec.exports;
}

var trait = function(spec) {
    return new Trait(spec);
};

trait.include = function(ctor, trait) {
    if (ctor === undefined || ctor === null) {
        throw new TypeError('The constructor to `include` must not be ' +
            'null or undefined.');
    }

    if (trait === undefined || trait === null) {
        throw new TypeError('The trait to `include` must not be ' +
            'null or undefined.');
    }

    if (!(trait instanceof Trait)) {
        throw new TypeError('The trait to `include` must be a trait.');
    }

    var proto = ctor.prototype;
    var requiredMethods = trait.required;
    for (var requiredKey in requiredMethods) {
        if (requiredMethods.hasOwnProperty(requiredKey)) {
            if (!proto[requiredKey] || (typeof proto[requiredKey] !== 'function')) {
                throw new Error('Required method [' + requiredKey + '] is missing');
            }
        }
    }

    var traitMethods = trait.exports;
    for (var traitKey in traitMethods) {
        if (traitMethods.hasOwnProperty(traitKey)) {
            proto[traitKey] = traitMethods[traitKey];
        }
    }
};

module.exports = trait;